# Copyright DB InfraGO AG and contributors
# SPDX-License-Identifier: Apache-2.0

from __future__ import annotations

import pytest

from raillabel.format import GpsImu
from raillabel.json_format import JSONCoordinateSystem, JSONStreamOther

# == Fixtures =========================


@pytest.fixture
def gps_imu_json(transform_json) -> tuple[JSONStreamOther, JSONCoordinateSystem]:
    return (
        JSONStreamOther(
            type="gps_imu",
            uri="/path/to/sensor/data",
            description="A very nice gps_imu",
        ),
        JSONCoordinateSystem(
            parent="base", type="sensor", pose_wrt_parent=transform_json, children=None
        ),
    )


@pytest.fixture
def gps_imu(transform) -> dict:
    return GpsImu(
        extrinsics=transform,
        uri="/path/to/sensor/data",
        description="A very nice gps_imu",
    )


# == Tests ============================


def test_from_json(gps_imu, gps_imu_json):
    actual = GpsImu.from_json(gps_imu_json[0], gps_imu_json[1])
    assert actual == gps_imu


def test_to_json(gps_imu, gps_imu_json):
    actual = gps_imu.to_json()
    assert actual == gps_imu_json


if __name__ == "__main__":
    pytest.main([__file__, "--disable-pytest-warnings", "--cache-clear", "-v"])



# Copyright DB InfraGO AG and contributors
# SPDX-License-Identifier: Apache-2.0
"""Package for the raillabel filter functionality."""

from .end_time_filter import EndTimeFilter
from .exclude_annotation_id_filter import ExcludeAnnotationIdFilter
from .exclude_annotation_type_filter import ExcludeAnnotationTypeFilter
from .exclude_attributes_filter import ExcludeAttributesFilter
from .exclude_frame_id_filter import ExcludeFrameIdFilter
from .exclude_object_id_filter import ExcludeObjectIdFilter
from .exclude_object_type_filter import ExcludeObjectTypeFilter
from .exclude_sensor_id_filter import ExcludeSensorIdFilter
from .exclude_sensor_type_filter import ExcludeSensorTypeFilter
from .include_annotation_id_filter import IncludeAnnotationIdFilter
from .include_annotation_type_filter import IncludeAnnotationTypeFilter
from .include_attributes_filter import IncludeAttributesFilter
from .include_frame_id_filter import IncludeFrameIdFilter
from .include_object_id_filter import IncludeObjectIdFilter
from .include_object_type_filter import IncludeObjectTypeFilter
from .include_sensor_id_filter import IncludeSensorIdFilter
from .include_sensor_type_filter import IncludeSensorTypeFilter
from .start_time_filter import StartTimeFilter

__all__ = [
    "IncludeFrameIdFilter",
    "ExcludeFrameIdFilter",
    "StartTimeFilter",
    "EndTimeFilter",
    "IncludeAnnotationIdFilter",
    "ExcludeAnnotationIdFilter",
    "IncludeAnnotationTypeFilter",
    "ExcludeAnnotationTypeFilter",
    "IncludeObjectIdFilter",
    "ExcludeObjectIdFilter",
    "IncludeObjectTypeFilter",
    "ExcludeObjectTypeFilter",
    "IncludeSensorIdFilter",
    "ExcludeSensorIdFilter",
    "IncludeSensorTypeFilter",
    "ExcludeSensorTypeFilter",
    "IncludeAttributesFilter",
    "ExcludeAttributesFilter",
]



# Copyright DB InfraGO AG and contributors
# SPDX-License-Identifier: Apache-2.0

from __future__ import annotations

import pytest

from raillabel.format import IntrinsicsPinhole
from raillabel.json_format import JSONIntrinsicsPinhole

# == Fixtures =========================


@pytest.fixture
def intrinsics_pinhole_json() -> dict:
    return JSONIntrinsicsPinhole(
        camera_matrix=[0.48, 0, 0.81, 0, 0, 0.16, 0.83, 0, 0, 0, 1, 0],
        distortion_coeffs=[0.49, 0.69, 0.31, 0.81, 0.99],
        width_px=2464,
        height_px=1600,
    )


@pytest.fixture
def intrinsics_pinhole() -> dict:
    return IntrinsicsPinhole(
        camera_matrix=(0.48, 0, 0.81, 0, 0, 0.16, 0.83, 0, 0, 0, 1, 0),
        distortion=(0.49, 0.69, 0.31, 0.81, 0.99),
        width_px=2464,
        height_px=1600,
    )


# == Tests ============================


def test_from_json(intrinsics_pinhole, intrinsics_pinhole_json):
    actual = IntrinsicsPinhole.from_json(intrinsics_pinhole_json)
    assert actual == intrinsics_pinhole


def test_to_json(intrinsics_pinhole, intrinsics_pinhole_json):
    actual = intrinsics_pinhole.to_json()
    assert actual == intrinsics_pinhole_json


if __name__ == "__main__":
    pytest.main([__file__, "--disable-pytest-warnings", "--cache-clear", "-v"])



# Copyright DB InfraGO AG and contributors
# SPDX-License-Identifier: Apache-2.0

from __future__ import annotations
import json

import pytest

from raillabel.format import Metadata
from raillabel.json_format import JSONMetadata

# == Fixtures =========================


@pytest.fixture
def metadata_json() -> JSONMetadata:
    return JSONMetadata(
        schema_version="1.0.0",
        name="some_file",
        subschema_version="4.0.0",
        exporter_version="1.2.3",
        file_version="0.1.5",
        tagged_file="path/to/data",
        annotator="John Doe",
        comment="this is a very nice annotation file",
    )


@pytest.fixture
def metadata() -> dict:
    return Metadata(
        schema_version="1.0.0",
        name="some_file",
        subschema_version="4.0.0",
        exporter_version="1.2.3",
        file_version="0.1.5",
        tagged_file="path/to/data",
        annotator="John Doe",
        comment="this is a very nice annotation file",
    )


# == Tests ============================


def test_from_json(metadata, metadata_json):
    actual = Metadata.from_json(metadata_json)
    assert actual == metadata


def test_to_json(metadata, metadata_json):
    actual = metadata.to_json()
    assert actual == metadata_json


def test_extra_fields():
    json_metadata = JSONMetadata(
        **{
            "schema_version": "1.0.0",
            "ADDITIONAL_STR": "SOME_VALUE",
            "ADDITIONAL_OBJECT": {"first_field": 2, "second_field": [1, 2, 3]},
        }
    )
    metadata = Metadata.from_json(json_metadata)
    json_metadata_again = metadata.to_json()

    actual = json.loads(json_metadata_again.model_dump_json(exclude_none=True))
    assert actual == {
        "schema_version": "1.0.0",
        "ADDITIONAL_STR": "SOME_VALUE",
        "ADDITIONAL_OBJECT": {"first_field": 2, "second_field": [1, 2, 3]},
    }


if __name__ == "__main__":
    pytest.main([__file__, "-v"])



# Copyright DB InfraGO AG and contributors
# SPDX-License-Identifier: Apache-2.0

from __future__ import annotations

from dataclasses import dataclass

from raillabel.format import Frame

from ._filter_abc import _FrameLevelFilter


@dataclass
class IncludeFrameIdFilter(_FrameLevelFilter):
    """Filter out all the frames in the scene that do NOT match a list of allowed ids."""

    frame_ids: set[int] | list[int]

    def passes_filter(self, frame_id: int, _: Frame) -> bool:
        """Assess if a frame passes this filter."""
        return frame_id in self.frame_ids



# Copyright DB InfraGO AG and contributors
# SPDX-License-Identifier: Apache-2.0

from __future__ import annotations

from typing import Literal

from ._json_format_base import _JSONFormatBase
from .frame_interval import JSONFrameInterval


class JSONElementDataPointer(_JSONFormatBase):
    """A pointer to element data of elements.

    It is indexed by 'name', and containing information about the element data type, for example,
    bounding box, cuboid, and the frame intervals in which this element_data exists within an
    element. That means, these pointers can be used to explore element data dynamic information
    within the JSON content.
    """

    attribute_pointers: dict[str, Literal["num", "text", "boolean", "vec"]]
    """This is a JSON object which contains pointers to the attributes of the element data pointed by
    this pointer. The attributes pointer keys shall be the 'name' of the attribute of the element
    data this pointer points to."""

    frame_intervals: list[JSONFrameInterval]
    "List of frame intervals of the element data pointed by this pointer."

    type: Literal["bbox", "num", "poly2d", "poly3d", "cuboid", "vec"]
    "Type of the element data pointed by this pointer."



# Copyright DB InfraGO AG and contributors
# SPDX-License-Identifier: Apache-2.0

from __future__ import annotations

from dataclasses import dataclass
from typing import Literal
from uuid import UUID

from raillabel.format import Bbox, Cuboid, Poly2d, Poly3d, Scene, Seg3d

from ._filter_abc import _AnnotationLevelFilter


@dataclass
class IncludeAnnotationTypeFilter(_AnnotationLevelFilter):
    """Filter out all annotations in the scene, that do NOT have the type (like bbox or cuboid)."""

    annotation_types: (
        set[Literal["bbox", "cuboid", "poly2d", "poly3d", "seg3d"]]
        | list[Literal["bbox", "cuboid", "poly2d", "poly3d", "seg3d"]]
    )

    def passes_filter(
        self, _: UUID, annotation: Bbox | Cuboid | Poly2d | Poly3d | Seg3d, __: Scene
    ) -> bool:
        """Assess if an annotation passes this filter."""
        annotation_type_str = None

        if isinstance(annotation, Bbox):
            annotation_type_str = "bbox"

        elif isinstance(annotation, Cuboid):
            annotation_type_str = "cuboid"

        elif isinstance(annotation, Poly2d):
            annotation_type_str = "poly2d"

        elif isinstance(annotation, Poly3d):
            annotation_type_str = "poly3d"

        elif isinstance(annotation, Seg3d):
            annotation_type_str = "seg3d"

        else:
            raise TypeError

        return annotation_type_str in self.annotation_types



# Copyright DB InfraGO AG and contributors
# SPDX-License-Identifier: Apache-2.0

from __future__ import annotations

from typing import Literal

from ._json_format_base import _JSONFormatBase
from .transform_data import JSONTransformData


class JSONCoordinateSystem(_JSONFormatBase):
    """A 3D reference frame."""

    parent: Literal["base", ""]
    "This is the string UID of the parent coordinate system this coordinate system is referring to."

    type: Literal["sensor", "local"]
    """This is a string that describes the type of the coordinate system, for example, 'local',
    'geo')."""

    pose_wrt_parent: JSONTransformData | None = None
    "The transformation with regards to the parent coordinate system."

    children: list[str] | None = None
    "List of children of this coordinate system."
